function main()

% Set parameters
p   = [0;0;1;0];					%
n   = 3000;                         %  
Nmc = 100;                           % # of MC replications
M   = 2000;                         % # of bootstrap replications
ki  = sqrt(log(log(n)));            %
alpha = 0.15;                       %
z = [-1 -1;                         % support of X
    -1 1                            %
    1 -1                            %
    1 1];                           %
Nz = size(z,1);                     %
Jeqraw = 2;                         % # of equalities (w/o conditioning)
Jineqraw = 2;                       % # of inequalities (w/o conditioning)
Jraw = [Jeqraw, Jineqraw];          %
Jeq  = Jeqraw*Nz;                   %
Jineq= Jineqraw*Nz;                 %
J = 2*Jeq + Jineq;                  %
lx = [-2 -2 -3 -3]; 				% lower bound of parameter space
ux = [2 2 0 0]; 					% upper bound of parameter space
                                    %
selp = 0.5;                 		% equilibrium selection probability
siglb = 1e-4;                       % lower bound on sigma
gpu = 'off';                         % set to 'on' if you use GPUs
cvxgen = 'done'; %'init';           % set to 'init' when you run this code for 1st time
parallel = 'on';					% set to 'on' if you use Parallel Computing Toolbox
                                    %
theta_true = [1;1;-1;-1]; 			% true pareter
d = length(theta_true);             %
Beta = 0.01;                        % amount of conservativeness for rho
rho = get_rho(Beta,J,d);            % 

mbase = 20;                         % 
minit = d*mbase + 1; 				% # of initial evaluation points
madd = mbase;                       % # of additional points to be sampled in each M-step
niter = 30;                         % maximum # of iterations

% Get the population upper bound
options_fmincon = optimoptions('fmincon','GradObj','on','GradConstr',...
    'on','Algorithm','interior-point');
nonlcon = @(theta)population_nonlcon(theta,theta_true,z,Jraw,selp);
[x1,fvl1,exitflag1]  = fmincon(@(theta)linobj(theta,p),[1;1;0;0],...              
    [],[],[],[],lx,ux,nonlcon,...                              
    options_fmincon); 
theta_U = -fvl1;


% Initialize Objects
stream = RandStream('mlfg6331_64','Seed',0);                     % initialize random number generator   
RandStream.setGlobalStream(stream)                               % 
                                                                 % 
addpath ./dace                                                   % use DACE kriging toolbox
dace_theta = 10*ones(1,d);                                       % set parameters for the A-step
dace_lob = 1e-1*ones(1,d);                                       % 
dace_upb = 10*ones(1,d);                                         % 
                                                                 % 
if strcmp(gpu,'on')                                              % 
    dev = gpuDevice();                                           % initialize GPUs
end                                                              % 
                                                                 % 
if strcmp(cvxgen,'init')                                         % 
urlwrite('http://b.cvxgen.com/download/cvxgen.m', 'cvxgen.m');   % complile CVXGEN codes
cvxgen(857543106560);                                            % 
end                                                              % 
                                                                 % 
solver = 'fmincon';                                              % 

% Data Generation %
[X1,X2,U1,U2,V] = drawsample(n,Nmc,selp);									% generate X, U, and V (used for eqbm seleciton)
beta1  = theta_true(1);                                                     % they are n x Nmc matrices
beta2  = theta_true(2);                                                     %
delta1 = theta_true(3);                                                     %
delta2 = theta_true(4);                                                     %
                                                                            %
if strcmp(gpu,'on')    
    outcomeFcn = @outcome;                                                  %
    GX1 = gpuArray(X1);                                                     %
    GX2 = gpuArray(X2);                                                     %
    GU1 = gpuArray(U1);                                                     %
    GU2 = gpuArray(U2);                                                     %
    GV  = gpuArray(V);                                                      %
    GY = arrayfun(outcomeFcn,GX1,GX2,GU1,GU2,GV,beta1,beta2,delta1,delta2); % generate all MC replications
    Y = gather(GY);                                                         % 
elseif strcmp(gpu,'off')                                                    %
    beta1  = repmat(beta1,n,Nmc);                                           %
    beta2  = repmat(beta2,n,Nmc);                                           %
    delta1 = repmat(delta1,n,Nmc);                                          %
    delta2 = repmat(delta2,n,Nmc);                                          %
    Y = arrayfun(@outcome,X1,X2,U1,U2,V,beta1,beta2,delta1,delta2);         %
end                                                                         %
phat = emp_freq(Y,X1,X2,Jraw,z);                                            % compute empirical frequencies 
phateq = phat(1:Jeq,:);                                                     % for equalities
phatineq = phat(Jeq+1:end,:);                                               % and for inequalities
                                                                            %
% Bootstrap everything %
if strcmp(parallel,'on')
opt = statset('UseParallel',true,'UseSubstreams',true);
emp_freqFcn = @(Y,X1,X2) emp_freq(Y,X1,X2,Jraw,z);
temp = bootstrp(M,emp_freqFcn,...
    Y,X1,X2,'Options',opt);                                                 % bootstrap data to get empirical freuencieis
elseif strcmp(paralle,'off')
temp = bootstrp(M,@(Y,X1,X2) emp_freq(Y,X1,X2,Jraw,z),...
	Y,X1,X2);				                                                % bootstrap data to get empirical freuencieis
end	

temp2 = reshape(temp,M,Jeq+Jineq,Nmc);
temp2 = permute(temp2,[2,1,3]);
temp2 = reshape(temp2,Jeq+Jineq,M*Nmc);
bdat = reshape(temp2,Jeq+Jineq,M*Nmc);                                      % 
bphateq = bdat(1:Jeq,:);                                                    % (Jeq x M) bootstrapped frequencies are tiled Nmc times
bphatineq = bdat(Jeq+1:end,:);                                              % (Jineq x M) bootstrapped frequencies are tiled Nmc times

% Obtain Std Devs
[sigeq, sigineq] =  get_sigma(phateq,phatineq);
sigeq = max(siglb,sigeq);       % regularize sigma if too close to 0
sigineq = max(siglb,sigineq); 
sig = [sigeq;sigineq];

% Obtain Inputs into LP that do not depend on theta
[ehat_eq,ehat_ineq,int_eq,int_ineq]...
    = get_moment(phateq,phatineq);
ehat = [ehat_eq;ehat_ineq];
int  = [int_eq;int_ineq];
[bhat_eq,bhat_ineq] = get_moment(bphateq,bphatineq);
Gmomeq = sqrt(n).*(bhat_eq - repelem(ehat_eq,1,M)).*repelem(int_eq,1,M);
Gmomineq = sqrt(n).*(bhat_ineq - repelem(ehat_ineq,1,M)).*repelem(int_ineq,1,M);
G = [Gmomeq./repelem(sigeq,1,M);Gmomineq./repelem(sigineq,1,M)];

% Initial Evaluation Points %
Theta_init = draw_points(minit,d,lx,ux);									% draw initial evaluation points
Theta_add = zeros(madd*niter,d);                                            % initialize matrix for additional points
C_add = zeros(madd*niter,1);                                                % initialize vector for cvs

% Beginning of MC
that = zeros(Nmc,1);					% vector to store endpoints of CI
steps = zeros(Nmc,1);					% vector to store # of steps until convergence
optim_flag = NaN(niter,Nmc);			% matrix to store exitflags
h_init = 1;                 		    % initial width of a band to which we add points
outer_tol = 1/(sqrt(n)*10);             % tolerance to exit iterations
for s=1:Nmc
%s = 2; % for now one MC draw
Gs = G(:,(s-1)*M+1:s*M);                                                 % extract G for this MC rep

thetaU_iter = zeros(niter,1);  % upper bound of CI from each iteration
te = zeros(niter,1);
ta = zeros(niter,1);
tm = zeros(niter,1);
flag_temp = NaN(niter,1);
for iter=1:niter
    % Set evaluation points
    if iter==1
        Theta = Theta_init;
    else
        Theta = Theta_add(madd*(iter-2)+1:madd*(iter-1),:);
    end
    
    m = length(Theta);
    C_temp = zeros(m,1);
    
    tic;
    parfor k = 1:m % loop over evaluation points
        theta = Theta(k,:)';
        
        % Compute objects that require theta
        [bound_eq,bound_ineq] = get_bound(theta,Jraw,z);
        sqnmom = sqrt(n).*[(ehat_eq - repmat(bound_eq,1,Nmc))./sigeq;            % compute studentized moment
            (ehat_ineq - repmat(bound_ineq,1,Nmc))./sigineq];                    %
        phi = gms(sqnmom,ki);                                                    % compute GMS
        %
        [Greq, Grineq] = get_gradient(theta,Jraw,z);                             %
        D = [repmat(Greq,1,Nmc)./repelem(sigeq,1,d).*repelem(int_eq,1,d);        % compute D
            repmat(Grineq,1,Nmc)./repelem(sigineq,1,d).*repelem(int_ineq,1,d)];  %
        %
        %
        Ds = D(:,(s-1)*d+1:s*d);                                                 % extract D for this MC rep
        phis = phi(:,s);                                                         % extract GMS for this MC rep
        C_B =  get_Bonferroni(Gs,alpha);                                         % get a Bonferroni cv bound
        %
        C_KMS = C_KMS_lin(Gs,Ds,phis,C_B,alpha,rho,p);  

        % get KMS cv
        C_temp(k) = C_KMS;                                                       % store it
    end                                                                          %
    te(iter)=toc;                                                                          %
    %
    if iter==1                                                                   %
        Theta_current = Theta_init;                                              % set points at which we got cvs
        C_init = C_temp;                                                         %
        C_current = C_temp;                                                      % set current cv values
    else                                                                         %
        C_add(madd*(iter-2)+1:madd*(iter-1)) = C_temp;                               % add additional evaluation points from last iteration
        Theta_current = [Theta_init;Theta_add(1:madd*(iter-1),:)];               % set current cv values
        C_current = [C_init;C_add(1:madd*(iter-1),:)];                           %
    end                                                                          %
    %
    tic                                                                          %
    [dmodel, ~] = dacefit(Theta_current,C_current,@regpoly0,@corrgauss,...       % fit a surrogate model
        dace_theta,dace_lob,dace_upb);                                           % dmodel contains a function to generate predicted values
    ta(iter)=toc;                                                                          %
    %
    ehats = ehat(:,s);                                                           %
    sigs  = sig(:,s);                                                            %
    ints  = int(:,s);                                                            %
    theta0 = theta_true;                                                         %
    %
    
    % Solve the optimization problem to obtain the (right) end point of CI
    tic
    if strcmp(solver,'fmincon')
        options = optimoptions('fmincon','GradObj','on','GradConstr','on','Algorithm','interior-point');
        [x2,fvl2,exitflag2] = fmincon(@(theta)linobj(theta,p),theta0,[],[],[],[],lx,ux,...
            @(theta)nonl_const(theta,dmodel,ehats,sigs,ints,n,Jraw,z),options);
        
    elseif strcmp(solver,'knitro')
        options = optimset('GradObj','on','GradConstr','on','Algorithm','interior-point');
        [x2,fvl2,exitflag2] = knitromatlab(@(theta)linobj(theta,p),theta0,[],[],[],[],lx,ux...
            ,@(theta)nonl_const(theta,dmodel,ehats,sigs,ints,n,Jraw,z),[],options);
    end
    tm(iter)=toc;
    flag_temp(iter)=exitflag2;
    thetaU_iter(iter,1) = -fvl2; % right end point of CI
    
    % Draw additional evaluation points
    if iter>1 && abs(thetaU_iter(iter)-thetaU_iter(iter-1)) < outer_tol
        break; % if the current optimal- previous optimal is within a tolerance, break
    else
        if exitflag2 >0
            h = h_init/iter;   % otherwise, set bandwidth
            temp = draw_points(madd,d,lx,ux,h,p,fvl2);
        elseif exitflag2 <= 0
            temp = draw_points(madd,d,lx,ux);
        end
        Theta_add(madd*(iter-1)+1:madd*iter,:) = temp;
    end
end
% This is the right end of CI
if flag_temp(iter) > 0
    that(s) = thetaU_iter(iter); 
else
    that(s) = -Inf;
end
steps(s) = iter;
optim_flag(:,s) = flag_temp; 
disp(['simulation ' num2str(s) ' done.'])
end

coverage = sum(theta_U<=that)/Nmc;
disp(coverage)
save temp.mat
plot(thetaU_iter(1:iter)) % plot to see if the process converged
ylim([-3,0])
disp('Time (in seconds) spent for E,A,M steps are')
disp([sum(te),sum(ta),sum(tm)])
disp('done')
end

function rho = get_rho(Beta,J,d)
% This function computes rho from Beta
% INPUTS
% Beta: amount of conservativeness
% OUTPUT
% rho

rho = -norminv((1-power(1-Beta,1/(d*nchoosek(J,d))))/2);
end


function [momeq, momineq] = population_freq(theta,z,Jraw,selp)
% This function computes population moments (pmom) if selp is nonempty
% If selp is nonempty, it computes the theoretical bounds (bound).
% The function assumes the following form:
% pmomeq - boundeq = 0
% pmomineq - boundeq <=0
% INPUTS
% theta: true parameter
% z: Nz-by-2, support of X
% Jraw: # of (raw) inequalities
% selp: selection probability
% OUTPUTS
% momeq: 1st term of population mom equality if selp is nonempty
%        2nd term (bound) of population mom equality if selp is empty
% momineq: similar to momeq (counterpart for inequality)

beta1  = theta(1);
beta2  = theta(2);
delta1 = theta(3);
delta2 = theta(4);

Nz = size(z,1);
Jeqraw = Jraw(1);
Jineqraw = Jraw(2);
Jeq = Jeqraw * Nz;
Jineq = Jineqraw * Nz;
momeq = zeros(Jeq,1);
momineq = zeros(Jineq,1);

for l = 1:Nz
    zval = z(l,:);
    momeq((l-1)*Jeqraw+1,1)   = normcdf(-zval(1)*beta1)...
        *normcdf(-zval(2)*beta2)/Nz;
    momeq((l-1)*Jeqraw+2,1)   = (1 - normcdf(-zval(1)*beta1-delta1))...
        *(1-normcdf(-zval(2)*beta2-delta2))/Nz;
    momineq((l-1)*Jineqraw+1,1) = normcdf(-zval(1)*beta1-delta1)...
        *(1-normcdf(-zval(2)*beta2))/Nz;
    momineq((l-1)*Jineqraw+2,1) = -(normcdf(-zval(1)*beta1-delta1)...
        *(1-normcdf(-zval(2)*beta2-delta2))...
    +normcdf(-zval(1)*beta1)...
    *(normcdf(-zval(2)*beta2-delta2)-normcdf(-zval(2)*beta2)))/Nz;
    if ~isempty(selp)
        width = momineq((l-1)*Jineqraw+1,1) + momineq((l-1)*Jineqraw+2,1);
        momineq((l-1)*Jineqraw+1,1) = momineq((l-1)*Jineqraw+1,1) - width*selp;
        momineq((l-1)*Jineqraw+2,1) = -momineq((l-1)*Jineqraw+1,1);
    end
end
end


function [c,ceq,GC,GCeq] = population_nonlcon(theta,theta_true,z,Jraw,selp)
% This function computes population moment constraints
% INPUTS
% theta: true parameter
% z: Nz-by-2, support of X
% Jraw: # of (raw) inequalities
% selp: selection probability
% OUTPUTS
% c: equality constraint
% ceq: inequality ocnstraint

d = length(theta);
Nz = size(z,1);
Jeq = Jraw(1)*Nz;
[pmomeq,pmomineq] = population_freq(theta_true,z,Jraw,selp);
pinteq = (abs(pmomeq)>1e-3).*(abs(pmomeq) < (1-1e-3));
pintineq = (abs(pmomineq)>1e-3).*(abs(pmomineq) < (1-1e-3));
[boundeq,boundineq] = population_freq(theta,z,Jraw,[]);
[Greq, Grineq] = get_gradient(theta,Jraw,z);
GC = (Grineq.*repmat(pintineq,1,d))';
GCeq = (Greq(1:Jeq,:).*repmat(pinteq,1,d))';

ceq = (pmomeq - boundeq).*pinteq;
c = (pmomineq - boundineq).*pintineq;
end


function [X1,X2,U1,U2,V] = drawsample(n,Nmc,selp)
% This function draws all samples across markets and MC replications
% INPUTS
% n: sample size
% Nmc: # of MC replications
% selp: selection probability
% OUTPUTS
% X1,X2: n x Nmc binary covariates
% U1,U2: n x Nmc latent variables
% V: n x Nmc latent binary variable (=1 w.p. selp)

U1 = randn(n,Nmc);
U2 = randn(n,Nmc);
V  = (rand(n,Nmc)>selp);
X1 = 2*randi(2,n,Nmc)-3;
X2 = 2*randi(2,n,Nmc)-3;
end


function [y] = outcome(x1,x2,u1,u2,v,beta1,beta2,delta1,delta2)
% This function generates outcomes for each market
% INPUTS
% x1,x2: scalar X1 and X2
% u1,u2: scalar u1 and u2
% beta1,...,delta2: true parameter
%
% OUTPUTS
% y: outcome, 0, 11, 1, or 10

y=NaN;
    % (1) unique equlibrium (0,0)
if (u1 < -x1*beta1 && u2 < -x2*beta2)
    y = 0;
    % (2) unique equlibirum (1,1)
elseif (u1 > -x1*beta1-delta1 && u2 > -x2*beta2-delta2)
    y = 11;
    
    % (3) unique eqilibirum (0,1)
elseif ( u1<-x1*beta1-delta1 && u2>-x2*beta2-delta2 )...
        || ( u1<-x1*beta1 &&  u2<-x2*beta2-delta2...
        && u2>-x2*beta2  )
    y = 01;
    % (4) multiple equlibrium region, select (0,1) with probability
    % (1-selp)
elseif (u1>-x1*beta1 && u1<-x1*beta1-delta1...
        && u2>-x2*beta2 && u2<-x2*beta2-delta2)
    if v==1
        y = 10;
    elseif v==0
        y = 01;
    end
else % unique eqlibirum  (1 0)
    y= 10;
end
end


function phat = emp_freq(Y,X1,X2,Jraw,z)
% This function coputes empirical frequencies
% INPUTS
% X1,X2: n x Nmc binary covariates
% z: Nz-by-2, support of X
% Jraw: # of (raw) inequalities
% OUTPUT
% phat: (2*Jeq + Jineq) x Nmc empirical frequencies
% Note: components corresponding to the 2nd raw mom ineq take negative
% values

[n,Nmc] = size(Y);
Nz = size(z,1);
Jeqraw = Jraw(1);
Jineqraw = Jraw(2);
Jeq = Jeqraw * Nz;
Jineq = Jineqraw * Nz;
phateq = zeros(Jeq,Nmc);
phatineq = zeros(Jineq,Nmc);

for l = 1:Nz
    zval = z(l,:);
    phateq((l-1)*Jeqraw+1,:)   = sum(Y==0  & X1 == zval(1) & X2 == zval(2))/n;
    phateq((l-1)*Jeqraw+2,:)   = sum(Y==11 & X1 == zval(1) & X2 == zval(2))/n;
    phatineq((l-1)*Jineqraw+1,:) = sum(Y==1  & X1 == zval(1) & X2 == zval(2))/n;
    phatineq((l-1)*Jineqraw+2,:) = -sum(Y==1 & X1 == zval(1) & X2 == zval(2))/n;
end
phat = [phateq; phatineq];
end


function [sigeq,sigineq] = get_sigma(phateq,phatineq)
% This function coputes estimates of standard deviations
% INPUTS
% phateq:   Jeq x Nmc empirical frequencies for equalities
% phatineq: Jineq x Nmc empirical frequencies for inequalities
% OUTPUT
% sigeq:   2*Jeq x Nmc std devs for equalities
% sigineq: Jineq x Nmc std devs for inequalities

sigeq   = (sqrt(phateq.*(1-phateq)));
sigeq   = [sigeq; sigeq];
sigineq = sqrt(abs(phatineq).*(1-abs(phatineq)));
end

function[bound_eq,bound_ineq] = get_bound(theta,Jraw,z)
% This function computes the 2nd term in moment (in)equalities.
% INPUTS
% theta: d x 1 parameter value
% Jraw: # of (raw) inequalities
% z: Nz-by-2, support of X
% OUTPUTS
% bound_eq: 2*Jeq x 1 bounds for equalities
% bound_ineq: Jineq x 1 bounds for inequalities

Nz = size(z,1);
Jeqraw = Jraw(1);
Jeq = Jeqraw * Nz;

bound_eq = zeros(Jeq*2,1);
[temp1,temp2] = population_freq(theta,z,Jraw,[]);
bound_eq(1:Jeq,:) =  temp1;
bound_eq(Jeq+1:end,:) = - temp1;
bound_ineq = temp2;
end


function [momeq,momineq,varargout] = get_moment(phateq,phatineq)
% This function coputes estimates of 1st term of sample moments
% INPUTS
% phateq:   Jeq x Nmc empirical frequencies for equalities
% phatineq: Jineq x Nmc empirical frequencies for inequalities
% OUTPUT
% momeq:   2*Jeq x Nmc sample moments for equalities
% momineq: Jineq x Nmc sample moments for inequalities
% varargout: contains indicators for moments being away the boundary

[Jeq,Nmc] = size(phateq);
momeq = zeros(Jeq*2,Nmc);

momeq(1:Jeq,:) = phateq;
momeq(Jeq+1:end,:) = -phateq;
momineq  = phatineq;

interior_eq = (abs(momeq)>1e-8).*(abs(momeq) < (1-1e-8));
interior_ineq = (abs(momineq)>1e-8).*(abs(momineq) < (1-1e-8));
varargout{1} = interior_eq;
varargout{2} = interior_ineq;
end


function [Geq, Gineq] = get_gradient(theta,Jraw,z)
% This function coputes gradients (not-studentized) of moments 
% INPUTS
% theta: true parameter
% z: Nz-by-2, support of X
% Jraw: # of (raw) inequalities
% OUTPUT
% Geq:   2*Jeq x d gradient for equalities
% Gineq: Jineq x d gradient for inequalities

d = length(theta);
beta1  = theta(1);
beta2  = theta(2);
delta1 = theta(3);
delta2 = theta(4);

Nz = size(z,1);
Jeqraw = Jraw(1);
Jineqraw = Jraw(2);
Jeq = Jeqraw * Nz;
Jineq = Jineqraw * Nz;
Geq = zeros(Jeq,d);
Gineq = zeros(Jineq,d);

for l = 1:Nz
    zval = z(l,:);
    x1t = zval(1); 
    x2t = zval(2);
    G1b1 = x1t*normpdf(-x1t*beta1)*normcdf(-x2t*beta2)/4;
    G1b2 = x2t*normcdf(-x1t*beta1)*normpdf(-x2t*beta2)/4;
    G1d1 = 0;
    G1d2 = 0;
    
    G2b1 = -x1t*normpdf(-x1t*beta1-delta1)*(1-normcdf(-x2t*beta2-delta2))/4;
    G2b2 = -x2t*(1-normcdf(-x1t*beta1-delta1))*normpdf(-x2t*beta2-delta2)/4;
    G2d1 = -normpdf(-x1t*beta1-delta1)*(1-normcdf(-x2t*beta2-delta2))/4;
    G2d2 = -(1-normcdf(-x1t*beta1-delta1))*normpdf(-x2t*beta2-delta2)/4;
    
    G3b1 = x1t*normpdf(-x1t*beta1-delta1)*(1-normcdf(-x2t*beta2))/4;
    G3b2 = -x2t*normcdf(-x1t*beta1-delta1)*normpdf(-x2t*beta2)/4;
    G3d1 = normpdf(-x1t*beta1-delta1)*(1-normcdf(-x2t*beta2))/4;
    G3d2 = 0;
    
    G4b1 = -x1t*normpdf(-x1t*beta1-delta1)*(1-normcdf(-x2t*beta2-delta2))/4 ...
        -x1t*normpdf(-x1t*beta1)*(normcdf(-x2t*beta2-delta2)-normcdf(-x2t*beta2))/4;
    G4b2 = x2t*normcdf(-x1t*beta1-delta1)*normpdf(-x2t*beta2-delta2)/4 ...
        +normcdf(-x1t*beta1)*(-x2t*normpdf(-x2t*beta2-delta2)+x2t*normpdf(-x2t*beta2))/4;
    G4d1 = -normpdf(-x1t*beta1-delta1)*(1-normcdf(-x2t*beta2-delta2))/4;
    G4d2 = normcdf(-x1t*beta1-delta1)*normpdf(-x2t*beta2-delta2)/4 ...
        +normcdf(-x1t*beta1)*(-normpdf(-x2t*beta2-delta2))/4;
    
    Geq((l-1)*Jeqraw+1,:) = [G1b1 G1b2 G1d1 G1d2];
    Geq((l-1)*Jeqraw+2,:) = [G2b1 G2b2 G2d1 G2d2];
    Gineq((l-1)*Jineqraw+1,:) = [G3b1 G3b2 G3d1 G3d2];
    Gineq((l-1)*Jineqraw+2,:) = [G4b1 G4b2 G4d1 G4d2];
end
Geq = [Geq; -Geq];
end


function phi = gms(momineq,ki)
% This function computes GMS
xi  = momineq/ki;   % GMS in Andrews Soares notation
phi = double((xi<-1));  
end

function C_B =  get_Bonferroni(Gs,alpha)
% This function computes a Bonferroni cv
C_B = quantile(max(Gs,[],2),1-alpha);
end



function C = C_KMS_lin(Gs,Ds,phis,C_B,alpha,rho,p)            %
%--------------------------------------------------------------------------%
% THIS FUNCTION SOLVES THE INNER LOOP PROBLEM AND CALIBRATES C FOR A GIVEN %
% THETA USING KMS                                                          %
%                                                                          %
% WBi: BOOTSTRAPED MOMENTS (CENTERED)                                      %
% VBi: SD OF  BOOTSTRAPED MOMENTS                                          %
% MOMS: WILL BE USED FOR MOMENT SELECTION UNDER GMS                        %
% C_AS: C(theta) from AS method                                            %
% PROBLEM BEING SOLVED TO CALIBRATE c                                      %
%                                                                          %
% Z^b_n(q,c,theta) = max q'lambda                                          %
% s.t. D_hat(theta)*lambda <= c ? G^b_n(theta) ? zeta_hat_n(theta)         %
% lambda in rho*B^d,                                                       %
%--------------------------------------------------------------------------%
%                                 %
%
M = size(Gs,2);
betaL = 0.5;                                                               %
CL = norminv(betaL);                                                       % Note that CL will be 0 as betaL is 0.5
CU = C_B;                                                                  %
C = (CL+CU)/2;                                                             %
                                                                           % extract those binding ones
%
TEMP = [zeros(M,1),ones(M,1)];                                             %

for k = 1:12;                                                              % caliberate beta
    [coverage,T] = get_coverage_LP(Gs,Ds,phis,C,TEMP,rho,p);               % compute coverage at c_k
    alpha_hat = 1-coverage;
    if abs(coverage - (1-alpha)) <= 1/M
        break;
    else
        if alpha_hat>alpha;                                                    % alpha_hat>alpha means coverage is less than desired, hence
            CL = C;                                                            % expand c: replace lower limit with C and search between C and CU
            C = (CL+CU)/2;                                                     %
            TEMP(:,1) = T(:,1);                                                %
        else                                                                   % If coverage is more than desired, then contract: restirct your
            CU = C;                                                            % search from CL to C
            C = (CL+CU)/2;                                                     %
            TEMP(:,2) = T(:,1);                                                %
        end
    end%
end                                                                        %
%--------------------------------------------------------------------------%
end

function [coverage, T] = get_coverage_LP(Gs,Ds,phis,C,TEMP,rho,p)
%--------------------------------------------------------------------------%
% THIS FUNCTION SOLVES THE INNER LOOP PROBLEM FOR A GIVEN C AND THETA      %
% INNER LOOP LINEAR PROBLEM:                                               %
% Z^b_n(q,c,theta) = max q'lambda                                          %
% s.t. D_hat(theta)*lambda <= c ? G^b_n(theta) ? zeta_hat_n(theta)         %
% lambda in rho*B^d,                                                       %
%                                                                          %
% INPUTS:                                                                  %
% WBi: BOOTSTRAPPED MOMENTS FOR iTH ITERATION                              %
% THETA                                                                    %
% D: DERIVATIVE MATRIX                                                     %
% phi: index of binding moments                                            %
% C:                                                                       %
% outputs are the coverage at the given caliberated value beta             %
%
%--------------------------------------------------------------------------%                                                     
% as max p*lambda s.t. A*lambda <= b
params=struct('A',{},'b',{},'p',{});                                       %

settings.verbose = 0;                                                      %
settings.eps = 1e-9;                                                       %
Jbind = sum(phis==0);   
[Jall,M] = size(Gs);
d = size(Ds,2);

K = [1:1:M]';                                                              %
idx_u = K((TEMP(:,2) - TEMP(:,1))==1);                                     % In the beginning idx_u would have all the rows                                  
mu = length(idx_u);                                                        %                                                    
that1_boot = zeros(mu,1);                                                  % this vector stacks 1st coordinate of optimal lambda                  
that2_boot = zeros(mu,1);                                                  % this vector stacks 2nd coordinate of optimal lambda                                                                       
nuhat_boot = zeros(mu,1);                                                  % this vector stacks maximized objective function in                                               
boot_GS_u = Gs(:,idx_u);                                                   %                                           
for j = 1:mu                                                               % prepare bootstarp variables
    W    = boot_GS_u(:,j);
    params(j).A = [Ds(phis==0,:)
        eye(d)
        -eye(d)
        zeros(Jall-Jbind,d)];
    params(j).b = [W((phis==0))+C
        rho*ones(2*d,1)
        ones(Jall-Jbind,1)];                                                                
    params(j).p=p;                                                         %
    [vars,status] = csolve(params(j),settings);                            % Using CVXGEN
    if status.converged==1                                                 %
        that1_boot(j) = vars.lambda(1);                                    %
        that2_boot(j) = vars.lambda(2);                                    %
        nuhat_boot(j) = status.optval;                                     %
    else                                                                   %
        that1_boot(j) = -Inf;                                              % otherwise, store NaNs
        that2_boot(j) = -Inf;                                              %
        nuhat_boot(j) = -Inf;                                              %
    end                                                                    %
end                                                                        %
%
T_u = zeros(M,1);                                                          % Stores bootstrap indices for which the solver converged
T_u((TEMP(:,2) + TEMP(:,1))==2) = 1;                                       % idx_u ignores these indices, hence update these
T_u(idx_u) = (nuhat_boot>=0);                                              %
coverage = sum((T_u==1))/M;
T = T_u;
end

function [c,ceq,GC,GCeq] = nonl_const(theta,dmodel,ehats,sigs,ints,n,Jraw,z)
% This function computes the nonlinear constraint for the projection
% problem
% INPUTS
% theta:  parameter
% dmodel: surrogate model (output from dace_fit function) for cv
% ehats:  J x 1, 1st term in the moment conditions
% sigs:   J x 1, std devs
% ints:   J x 1, indicator for moments being away from boundary
% n:      scalar, sample size
% z:      Nz-by-2, support of X
% Jraw:   # of (raw) inequalities
% OUTPUT
% c:      inequality constraint
% GC:     gradient

d = size(theta,1);
ceq = 0;
GCeq = zeros(d,1);

Nz = size(z,1);
Jeqraw = Jraw(1);
Jineqraw = Jraw(2);
Jeq = Jeqraw * Nz;
Jineq = Jineqraw * Nz;
J = 2*Jeq + Jineq;

[bound_eq,bound_ineq] = get_bound(theta,Jraw,z);
[Greq, Grineq] = get_gradient(theta,Jraw,z);
Gr = [Greq; Grineq];
Ds = Gr./repmat(sigs,1,d);
bound = [bound_eq;bound_ineq];
moms = (ehats-bound)./sigs;
[chat,cgrad] = predictor(theta,dmodel);
c = sqrt(n)*moms.*ints-repmat(chat,J,1);
GC = sqrt(n)*Ds.*repmat(ints,1,d) - repmat(cgrad',J,1);
GC = GC';
end

function [f,g] = linobj(theta,p)
% This function computes the objective function for the projection
% problem
% INPUTS
% theta: parameter
% p:     direction of interest
% OUTPUT
% f:     function value
% g:     gradient

f = -p'*theta;

if nargout > 1
    g = -p;
end
end

function points = draw_points(madd,d,lx,ux,varargin)
% This function draws points on parameter space
% INPUTS
% madd: # of (additional) evaluation points
% d: dimension of theta
% lx: 1xd lower bound on parameter space
% ux: 1xd upper bound on parameter space
% varargin: additional arguments. If provided, the function draws points that achieve values close to fvl2.
% OUTPUTS
% points: madd x d evaluation points
	
  if nargin > 4   % if h, p, fvl2 are passed from outside, draw locally
    h = varargin{1};
    p = varargin{2};
    fvl2 = varargin{3};
    temp = lhsamp(madd,d);                                                    % draw points in a cube
    pone = find(p~=0);                                                        %
    pzero = find(p==0);                                                       %
    temp(:,pone) = -fvl2 + h*(temp(:,pone)-mean(temp(:,pone)));               % make the coordinate of interest around the current optimal
    temp(:,pzero) = repmat(ux(pzero)-lx(pzero),madd,1).*temp(:,pzero)...      % rescale other coordinates
        +repmat(lx(pzero),madd,1);     
  elseif nargin == 4   % otherwise, draw globally
    temp = lhsamp(madd,d);                                                    % draw points on a cube (Latin hypercube sampling)
    temp = repmat((ux-lx),madd,1).*temp+repmat(lx,madd,1);					  % rescale to draw points on parameter space	
  end
points = temp;
end


function A = repelem(A, varargin)

  index = cell(1, nargin-1);
  for iDim = 1:nargin-1
    lens = varargin{iDim};
    if isscalar(lens)
      if (lens == 1)
        index{iDim} = ':';
        continue
      else
        lens = repmat(lens, 1, size(A, iDim));
      end
    end
    vals = 1:size(A, iDim);
    clens = cumsum(lens);
    index{iDim} = zeros(1, clens(end));
    index{iDim}([1 clens(1:end-1)+1]) = diff([0 vals]);
    index{iDim} = cumsum(index{iDim});
  end
  A = A(index{:});

end
